@@ -86,16 +86,18 @@ logLikFactory <- function(Y, X=0, weights, bhazard, dlist,
         dargs$log <- TRUE
         logdens <- do.call(dfns$d, dargs)
         
-        ## Left censoring times
-        pmaxargs <- fnargs.nevent
-        pmaxargs$q <- left.censor # Inf if right-censored, giving pmax=1
-        pmax <- do.call(dfns$p, pmaxargs)
-        pmax[pmaxargs$q==Inf] <- 1  # in case user-defined function doesn't already do this
-        
-        ## Right censoring times
-        pargs <- fnargs.nevent
-        pargs$q <- right.censor
-        pmin <- do.call(dfns$p, pargs)
+        ## Left censoring times (upper bound for event time) 
+        if (any(!event)){
+            pmaxargs <- fnargs.nevent
+            pmaxargs$q <- left.censor # Inf if right-censored, giving pmax=1
+            pmax <- do.call(dfns$p, pmaxargs)
+            pmax[pmaxargs$q==Inf] <- 1  # in case user-defined function doesn't already do this
+            
+        ## Right censoring times (lower bound for event time) 
+            pargs <- fnargs.nevent
+            pargs$q <- right.censor
+            pmin <- do.call(dfns$p, pargs)
+        }
         
         ## Left-truncation
         targs   <- fnargs
@@ -114,7 +116,8 @@ logLikFactory <- function(Y, X=0, weights, bhazard, dlist,
         }
         ## Express as vector of individual likelihood contributions
         loglik[event] <- (logdens*event.weights) + offseti
-        loglik[!event] <- (log(pmax - pmin)*no.event.weights)
+        if (any(!event))
+            loglik[!event] <- (log(pmax - pmin)*no.event.weights)
         loglik <- loglik - log(pobs)*weights
         
         ret <- -sum(loglik)
@@ -240,6 +243,18 @@ expand.summfn.args <- function(summfn){
     summfn2
 }
 
+### On entry:
+### event (status=1)            time1=event time
+### right-censoring (status=0)  time1=lower bound
+### left-censoring (status=2)   time1=upper bound 
+### interval-censoring (status=3)  time1=lower, time2=upper
+
+### On exit
+### time1=lower bound or event time
+### time2=upper bound
+### start=left truncation time
+### so meaning of time1,time2 reversed with left-censoring
+
 check.flexsurv.response <- function(Y){
     if (!inherits(Y, "Surv"))
         stop("Response must be a survival object")
@@ -249,7 +264,8 @@ check.flexsurv.response <- function(Y){
         Y <- cbind(Y, time=Y[,"stop"] - Y[,"start"], time1=Y[,"stop"], time2=Inf)
     else if (attr(Y, "type") == "interval"){
         Y[,"time2"][Y[,"status"]==0] <- Inf   # upper bound with right censoring 
-        Y[,"time2"][Y[,"status"]==2] <- -Inf  # 
+        Y[,"time2"][Y[,"status"]==2] <- Y[,"time1"][Y[,"status"]==2]
+        Y[,"time1"][Y[,"status"]==2] <- 0  # 
         Y <- cbind(Y, start=0, stop=Y[,"time1"], time=Y[,"time1"])
     }
     else if (attr(Y, "type") == "right")
@@ -471,7 +487,9 @@ compress.model.matrices <- function(mml){
 ##' \code{\link{confint}} methods for \code{flexsurvreg} objects work on this
 ##' scale.} \item{coefficients}{The transformed maximum likelihood estimates,
 ##' as in \code{res.t}. Calling \code{coef()} on a \code{\link{flexsurvreg}}
-##' object simply returns this component.} \item{loglik}{Log-likelihood}
+##' object simply returns this component.} \item{loglik}{Log-likelihood. This will differ from Stata, where the sum of
+##' the log uncensored survival times is added to the log-likelihood in
+##' survival models, to remove dependency on the time scale.}
 ##' \item{logliki}{Vector of individual contributions to the log-likelihood}
 ##' \item{AIC}{Akaike's information criterion (-2*log likelihood + 2*number of
 ##' estimated parameters)} \item{cov}{Covariance matrix of the parameters, on
@@ -508,7 +526,7 @@ compress.model.matrices <- function(mml){
 ##' or alternative values for each parameter) and return the results as a
 ##' vector.  The function \code{\link{Vectorize}} may be helpful for doing
 ##' this: see the example below.
-##' 
+
 ##' These functions may be in an add-on package (see below for an example) or
 ##' may be user-written.  If they are user-written they must be defined in the
 ##' global environment, or supplied explicitly through the \code{dfns} argument
@@ -771,7 +789,7 @@ flexsurvreg <- function(formula, anc=NULL, data, weights, bhazard, subset, na.ac
     if (missing(inits) && is.null(dlist$inits))
         stop("\"inits\" not supplied, and no function to estimate them found in the custom distribution list")
     if (missing(inits) || any(is.na(inits))){
-        yy <- ifelse(Y[,"status"]==3 & is.finite(Y[,"time2"]), (Y[,"time1"] + Y[,"time2"])/2, Y[,"time"])
+        yy <- ifelse(Y[,"status"]==3 & is.finite(Y[,"time2"]), (Y[,"time1"] + Y[,"time2"])/2, Y[,"time1"])
         wt <- yy*weights*length(yy)/sum(weights)
         dlist$inits <- expand.inits.args(dlist$inits)
         inits.aux <- c(aux, list(forms=forms, data=if(missing(data)) NULL else data, weights=temp$weights,
@@ -969,617 +987,6 @@ form.model.matrix <- function(object, newdata){
 }
 
 
-
-##' Summaries of fitted flexible survival models
-##' 
-##' Return fitted survival, cumulative hazard or hazard at a series of times
-##' from a fitted \code{\link{flexsurvreg}} or \code{\link{flexsurvspline}}
-##' model.
-##' 
-##' Time-dependent covariates are not currently supported.  The covariate
-##' values are assumed to be constant through time for each fitted curve.
-##' 
-##' @param object Output from \code{\link{flexsurvreg}} or
-##' \code{\link{flexsurvspline}}, representing a fitted survival model object.
-##' @param newdata Data frame containing covariate values to produce fitted
-##' values for.  Or a list that can be coerced to such a data frame.  There
-##' must be a column for every covariate in the model formula, and one row for
-##' every combination of covariates the fitted values are wanted for.  These
-##' are in the same format as the original data, with factors as a single
-##' variable, not 0/1 contrasts.
-##' 
-##' If this is omitted, if there are any continuous covariates, then a single
-##' summary is provided with all covariates set to their mean values in the
-##' data - for categorical covariates, the means of the 0/1 indicator variables
-##' are taken.  If there are only factor covariates in the model, then all
-##' distinct groups are used by default.
-##' @param X Alternative way of defining covariate values to produce fitted
-##' values for.  Since version 0.4, \code{newdata} is an easier way that
-##' doesn't require the user to create factor contrasts, but \code{X} has been
-##' kept for backwards compatibility.
-##' 
-##' Columns of \code{X} represent different covariates, and rows represent
-##' multiple combinations of covariate values.  For example
-##' \code{matrix(c(1,2),nrow=2)} if there is only one covariate in the model,
-##' and we want survival for covariate values of 1 and 2.  A vector can also be
-##' supplied if just one combination of covariates is needed.
-##' 
-##' For ``factor'' (categorical) covariates, the values of the contrasts
-##' representing factor levels (as returned by the \code{\link{contrasts}}
-##' function) should be used.  For example, for a covariate \code{agegroup}
-##' specified as an unordered factor with levels \code{20-29, 30-39, 40-49,
-##' 50-59}, and baseline level \code{20-29}, there are three contrasts.  To
-##' return summaries for groups \code{20-29} and \code{40-49}, supply \code{X =
-##' rbind(c(0,0,0), c(0,1,0))}, since all contrasts are zero for the baseline
-##' level, and the second contrast is ``turned on'' for the third level
-##' \code{40-49}.
-##' @param type \code{"survival"} for survival probabilities.
-##' 
-##' \code{"cumhaz"} for cumulative hazards.
-##' 
-##' \code{"hazard"} for hazards.
-##' 
-##' \code{"rmst"} for restricted mean survival.
-##' 
-##' \code{"mean"} for mean survival.
-##' 
-##' Ignored if \code{"fn"} is specified.
-##' @param fn Custom function of the parameters to summarise against time.
-##' This has optional first two arguments \code{t} representing time, and
-##' \code{start} representing left-truncation points, and any remaining
-##' arguments must be parameters of the distribution.  It should return a
-##' vector of the same length as \code{t}.
-##' @param t Times to calculate fitted values for. By default, these are the
-##' sorted unique observation (including censoring) times in the data - for
-##' left-truncated datasets these are the "stop" times.
-##' @param start Optional left-truncation time or times.  The returned
-##' survival, hazard or cumulative hazard will be conditioned on survival up to
-##' this time.
-##' 
-##' A vector of the same length as \code{t} can be supplied to allow different
-##' truncation times for each prediction time, though this doesn't make sense
-##' in the usual case where this function is used to calculate a predicted
-##' trajectory for a single individual.  This is why the default \code{start}
-##' time was changed for version 0.4 of \pkg{flexsurv} - this was previously a
-##' vector of the start times observed in the data.
-##' @param ci Set to \code{FALSE} to omit confidence intervals.
-##' @param B Number of simulations from the normal asymptotic distribution of
-##' the estimates used to calculate confidence intervals.  Decrease for greater
-##' speed at the expense of accuracy, or set \code{B=0} to turn off calculation
-##' of CIs.
-##' @param cl Width of symmetric confidence intervals, relative to 1.
-##' @param tidy If \code{TRUE}, then the results are returned as a tidy data
-##' frame instead of a list.  This can help with using the \pkg{ggplot2}
-##' package to compare summaries for different covariate values.
-##' @param ... Further arguments passed to or from other methods.  Currently
-##' unused.
-##' @return If \code{tidy=FALSE}, a list with one component for each unique
-##' covariate value (if there are only categorical covariates) or one component
-##' (if there are no covariates or any continuous covariates).  Each of these
-##' components is a matrix with one row for each time in \code{t}, giving the
-##' estimated survival (or cumulative hazard, or hazard) and 95\% confidence
-##' limits.  These list components are named with the covariate names and
-##' values which define them.
-##' 
-##' If \code{tidy=TRUE}, a data frame is returned instead.  This is formed by
-##' stacking the above list components, with additional columns to identify the
-##' covariate values that each block corresponds to.
-##' 
-##' If there are multiple summaries, an additional list component named
-##' \code{X} contains a matrix with the exact values of contrasts (dummy
-##' covariates) defining each summary.
-##' 
-##' The \code{\link{plot.flexsurvreg}} function can be used to quickly plot
-##' these model-based summaries against empirical summaries such as
-##' Kaplan-Meier curves, to diagnose model fit.
-##' 
-##' Confidence intervals are obtained by random sampling from the asymptotic
-##' normal distribution of the maximum likelihood estimates (see, e.g. Mandel
-##' (2013)).
-##' @author C. H. Jackson \email{chris.jackson@@mrc-bsu.cam.ac.uk}
-##' @seealso \code{\link{flexsurvreg}}, \code{\link{flexsurvspline}}.
-##' @references Mandel, M. (2013). "Simulation based confidence intervals for
-##' functions with complicated derivatives." The American Statistician (in
-##' press).
-##' @keywords models
-##' @export
-summary.flexsurvreg <- function(object, newdata=NULL, X=NULL, type="survival", fn=NULL,
-                                t=NULL, start=0, ci=TRUE, B=1000, cl=0.95, tidy=FALSE,
-                                ...)
-{
-    x <- object
-    dat <- x$data
-    Xraw <- model.frame(x)[,unique(attr(model.frame(x),"covnames.orig")),drop=FALSE]
-    isfac <- sapply(Xraw, function(x){is.factor(x) || is.character(x)})
-    type <- match.arg(type, c("survival","cumhaz","hazard","rmst","mean","median"))
-    if (is.null(newdata)){
-        if (is.vector(X)) X <- matrix(X, nrow=1)
-        if (x$ncovs > 0 && is.null(X)) {
-            ## if any continuous covariates, calculate fitted survival for "average" covariate value
-            if (!all(isfac)){
-                nd <- colMeans(model.matrix(x))
-                X <- matrix(nd ,nrow=1, dimnames=list(NULL,names(nd)))
-                attr(X, "newdata") <- as.data.frame(X)
-            }
-            ## else calculate for all different factor groupings
-            else {
-                X <- unique(model.matrix(x))
-                ## build names like "COVA=value1,COVB=value2"
-                nam <- as.matrix(unique(Xraw))
-                for (i in 1:ncol(nam)) nam[,i] <- paste(colnames(nam)[i], nam[,i], sep="=")
-                rownames(X) <- apply(nam, 1, paste, collapse=",")
-                attr(X, "newdata") <- unique(Xraw)
-            }
-        }
-        else if (is.null(X)) X <- as.matrix(0, nrow=1, ncol=max(x$ncoveffs,1))
-        else if (!is.matrix(X) || (is.matrix(X) && ncol(X) != x$ncoveffs)) {
-            plural <- if (x$ncoveffs > 1) "s" else ""
-            stop("expected X to be a matrix with ", x$ncoveffs, " column", plural, " or a vector with ", x$ncoveffs, " element", plural)
-        }
-        else {
-            attr(X, "newdata") <- X
-            colnames(attr(X, "newdata")) <- colnames(model.matrix(x))
-        }
-    } else
-        X <- form.model.matrix(object, as.data.frame(newdata))
-    
-    if(type == "mean"){
-      if(!is.null(t)) warning("Mean selected, but time specified.  For restricted mean, set type to 'rmst'.")
-      # Type = mean same as RMST w/ time = Inf
-      t = rep(Inf,length(start))
-    }
-    else if(type == "median"){
-      if(!is.null(t)) warning("Median selected, but time specified.")
-      t = rep(0.5,length(start))
-    }
-    else if(type == "rmst"){
-        if (is.null(t))
-            t <- max(dat$Y[,"time1"])
-    }
-    else if (is.null(t))
-        t <- sort(unique(dat$Y[,"stop"]))
-    if (length(start)==1)
-        start <- rep(start, length(t))
-    else if (length(start) != length(t))
-        stop("length of \"start\" is ",length(start),". Should be 1, or length of \"t\" which is ",length(t))
-
-    if (is.null(fn)) {
-        fn <- summary.fns(x, type)
-    }
-    fn <- expand.summfn.args(fn)
-    fncall <- list(t,start)
-    beta <- if (x$ncovs==0) 0 else x$res[x$covpars,"est"]
-    if (ncol(X) != length(beta)){
-        ## don't think we should ever reach here - error should be caught in newdata or X
-        isare <- if(length(beta)==1) "is" else "are"
-        plural <- if(ncol(X)==1) "" else "s"
-        pluralc <- if(length(beta)==1) "" else "s"
-        stop("Supplied X has ", ncol(X), " column",plural," but there ",isare," ",
-             length(beta), " covariate effect", pluralc)
-    }
-    dlist <- x$dlist
-    ret <- vector(nrow(X), mode="list")
-    if(!is.null(newdata)){
-        nd <- attr(X, "newdata")
-        covnames <- apply(as.data.frame(nd), 1, function(x)paste0(names(nd), "=", x, collapse=", "))
-    }
-    else covnames <- rownames(X)
-    names(ret) <- covnames
-    for (i in 1:nrow(X)) {
-        basepars.mat <- add.covs(x, x$res.t[dlist$pars,"est"], beta, X[i,,drop=FALSE], transform=FALSE)
-        basepars <- as.list(as.data.frame(basepars.mat))
-        fncall[dlist$pars] <- basepars
-        for (j in seq_along(x$aux)){
-            fncall[[names(x$aux)[j]]] <- x$aux[[j]]
-        }
-        y <- do.call(fn, fncall)
-        if (ci){
-            res.ci <- cisumm.flexsurvreg(x, t, start, X[i,,drop=FALSE], fn=fn, B=B, cl=cl)
-            ly <- res.ci[,1]
-            uy <-  res.ci[,2]
-        }
-        if(type %in% c("median","mean")) ret[[i]] <- data.frame(est=y, row.names=NULL)
-        else ret[[i]] <- data.frame(time=t, est=y, row.names=NULL)
-        if (ci) { ret[[i]]$lcl <- ly; ret[[i]]$ucl <- uy}
-    }
-    if (x$ncovs>0) attr(ret,"X") <- X
-    if (tidy) {
-        ret <- do.call("rbind", ret)
-        if (x$ncovs>0) {
-            nd <- attr(X, "newdata")
-            covdf <- nd[rep(seq_len(nrow(nd)), each=length(t)), , drop=FALSE]
-            rownames(ret) <- rownames(covdf) <- NULL
-            ret <- cbind(ret, covdf)
-        }
-    }
-    class(ret) <- c("summary.flexsurvreg",class(ret))
-    ret
-}
-
-summary.fns <- function(x, type){
-    switch(type,   # TODO warn for clashing arguments in dfns
-           "survival" = function(t,start,...) {
-               ret <- (1 - x$dfns$p(t,...))/(1 - x$dfns$p(start,...))
-               ret[t<start] <- 1 # prob[t<start] was previously 0
-               ret
-           },
-           "median" = function(start,...) {
-             start_p = 1 - x$dfns$p(start,...)
-             med_from_start = start_p/2
-             ret = x$dfns$q(med_from_start,...)
-           },
-           "hazard" = function(t,start,...) {
-               ret <- x$dfns$h(t,...) * (1 - x$dfns$p(start,...))
-               ret[t<start] <- 0
-               ret
-           },
-           "cumhaz" = function(t,start,...) {
-               ret <- x$dfns$H(t,...) - x$dfns$H(start,...)
-               ret[t<start] <- 0
-               ret
-           },
-           "rmst" = function(t,start,...) x$dfns$rmst(t,start=start, ...),
-           "mean" = function(t,start,...) x$dfns$mean(...)
-    )
-}
-
-##' @export
-print.summary.flexsurvreg <- function(x, ...){
-    if (!inherits(x, "data.frame")){
-        for (i in seq_along(x)){
-            cat(names(x)[i], "\n")
-            print(x[[i]])
-            if (i<length(x)) cat("\n")
-        }
-    } else print.data.frame(x)
-}
-
-## TODO would converting newdata to X be better handled in this function
-
-add.covs <- function(x, pars, beta, X, transform=FALSE){  ## TODO option to transform on input
-    nres <- nrow(X)
-    if (!is.matrix(pars)) pars <- matrix(pars, nrow=nres, ncol=length(pars), byrow=TRUE)
-    if (!is.matrix(beta)) beta <- matrix(beta, nrow=1)
-    for (j in seq(along=x$dlist$pars)){
-        covinds <- x$mx[[x$dlist$pars[j]]]
-        if (length(covinds) > 0){
-            pars[,j] <- pars[,j] + beta[,covinds] %*% t(X[,covinds,drop=FALSE])
-        }
-        if (!transform)
-            pars[,j] <- x$dlist$inv.transforms[[j]](pars[,j])
-    }
-    colnames(pars) <- x$dlist$pars
-    pars
-}
-
-## Draw B samples from multivariate normal distribution of baseline
-## parameter estimators, for given covariate values
-
-
-
-##' Simulate from the asymptotic normal distribution of parameter estimates.
-##' 
-##' Produce a matrix of alternative parameter estimates under sampling
-##' uncertainty, at covariate values supplied by the user.  Used by
-##' \code{\link{summary.flexsurvreg}} for obtaining confidence intervals around
-##' functions of parameters.
-##' 
-##' 
-##' @param x A fitted model from \code{\link{flexsurvreg}}.
-##' @param B Number of samples.
-##' @param newdata Data frame or list containing the covariate values to
-##' evaluate the parameters at.  If there are covariates in the model, at least
-##' one of \code{newdata} or \code{X} must be supplied, unless \code{raw=TRUE}.
-##' @param X Alternative (less convenient) format for covariate values: a
-##' matrix with one row, with one column for each covariate or factor contrast.
-##' Formed from all the "model matrices", one for each named parameter of the
-##' distribution, with intercepts excluded, \code{cbind}ed together.
-##' @param transform \code{TRUE} if the results should be transformed to the
-##' real-line scale, typically by log if the parameter is defined as positive.
-##' The default \code{FALSE} returns parameters on the natural scale.
-##' @param raw Return samples of the baseline parameters and the covariate
-##' effects, rather than the default of adjusting the baseline parameters for
-##' covariates.
-##' @return If \code{newdata} includes only one covariate combination, a matrix
-##' will be returned with \code{B} rows, and one column for each named
-##' parameter of the survival distribution.
-##' 
-##' If more than one covariate combination is requested (e.g. \code{newdata} is
-##' a data frame with more than one row), then a list of matrices will be
-##' returned, one for each covariate combination.
-##' @author C. H. Jackson \email{chris.jackson@@mrc-bsu.cam.ac.uk}
-##' @seealso \code{\link{summary.flexsurvreg}}
-##' @references Mandel, M. (2013). "Simulation based confidence intervals for
-##' functions with complicated derivatives." The American Statistician (in
-##' press).
-##' @keywords models
-##' @examples
-##' 
-##'     fite <- flexsurvreg(Surv(futime, fustat) ~ age, data = ovarian, dist="exp")
-##'     normboot.flexsurvreg(fite, B=10, newdata=list(age=50))
-##'     normboot.flexsurvreg(fite, B=10, X=matrix(50,nrow=1))
-##'     normboot.flexsurvreg(fite, B=10, newdata=list(age=0))  ## closer to...
-##'     fite$res
-##' @export
-normboot.flexsurvreg <- function(x, B, newdata=NULL, X=NULL, transform=FALSE, raw=FALSE){
-    if (x$ncovs > 0 && !raw) {
-        if (is.null(X)) {
-            if (is.null(newdata)) stop("neither \"newdata\" nor \"X\" supplied")
-            X <- form.model.matrix(x, as.data.frame(newdata))
-        }
-    } else X <- as.matrix(0, nrow=1, ncol=1)
-    sim <- matrix(nrow=B, ncol=nrow(x$res))
-    colnames(sim) <- rownames(x$res)
-    if (is.na(x$cov[1])) stop("Covariance matrix not available from non-converged model")
-    sim[,x$optpars] <- rmvnorm(B, x$opt$par, x$cov)
-    sim[,x$fixedpars] <- rep(x$res.t[x$fixedpars,"est"],each=B)
-    if (x$ncovs > 0 && !raw){
-        beta <- sim[, x$covpars, drop=FALSE]
-        if (nrow(X)==1){
-            res <- sim[,x$dlist$pars,drop=FALSE]
-            res <- add.covs(x=x, pars=res, beta=beta, X=X, transform=transform)
-        }  else {
-            res <- vector(nrow(X), mode="list")
-            for (i in 1:nrow(X)) {
-                res[[i]] <- sim[,x$dlist$pars,drop=FALSE]
-                res[[i]] <- add.covs(x=x, pars=res[[i]], beta=beta, X=X[i,,drop=FALSE], transform=transform)
-            }
-        }
-    } else {
-        res <- sim
-        if (!transform){
-            for (j in seq(along=x$dlist$pars)){
-                res[,j] <- x$dlist$inv.transforms[[j]](res[,j])
-            }
-        }
-    }
-    attr(res, "X") <- X
-    res
-}
-
-### Compute CIs for survival, cumulative hazard or hazard at supplied
-### times t and covariates X, using random sample of size B from the
-### assumed MVN distribution of MLEs.
-
-normbootfn.flexsurvreg <- function(x, t, start, newdata=NULL, X=NULL, fn, B){
-    sim <- normboot.flexsurvreg(x, B, newdata=newdata, X=X)
-    X <- attr(sim, "X")
-    if (!is.list(sim)) sim <- list(sim)
-    ret <- array(NA_real_, dim=c(nrow(X), B, length(t)))
-    for (k in 1:nrow(X)){
-        for (i in seq(length=B)) {
-            fncall <- list(t,start)
-            for (j in seq(along=x$dlist$pars))
-                fncall[[x$dlist$pars[j]]] <- sim[[k]][i,j]
-            for (j in seq_along(x$aux))
-                fncall[[names(x$aux)[j]]] <- x$aux[[j]]
-            ret[k,i,] <- do.call(fn, fncall)
-        }
-    }
-    if (nrow(X)==1) ret[1,,,drop=FALSE] else ret
-}
-
-cisumm.flexsurvreg <- function(x, t, start, X, fn, B=1000, cl=0.95) {
-    if (any(is.na(x$res[,2])) || (B==0))
-        ret <- array(NA, dim=c(length(t), 2))
-    else {
-        ret <- normbootfn.flexsurvreg(x=x, t=t, start=start, X=X, fn=fn, B=B)
-        ret <- apply(ret, c(1,3), function(x)quantile(x, c((1-cl)/2, 1 - (1-cl)/2), na.rm=TRUE))
-        ret <- t(ret[,1,])
-    }
-    ret
-}
-
-
-
-##' Plots of fitted flexible survival models
-##' 
-##' Plot fitted survival, cumulative hazard or hazard from a parametric model
-##' against nonparametric estimates to diagnose goodness-of-fit.  Alternatively
-##' plot a user-defined function of the model parameters against time.
-##' 
-##' 
-##' @param x Output from \code{\link{flexsurvreg}} or
-##' \code{\link{flexsurvspline}}, representing a fitted survival model object.
-##' @param newdata Data frame containing covariate values to produce fitted
-##' values for.  See \code{\link{summary.flexsurvreg}}.
-##' 
-##' If there are only factor covariates in the model, then Kaplan-Meier (or
-##' nonparametric hazard...)  curves are plotted for all distinct groups, and
-##' by default, fitted curves are also plotted for these groups.  To plot
-##' Kaplan-Meier and fitted curves for only a subset of groups, use
-##' \code{plot(survfit())} followed by \code{lines.flexsurvreg()}.
-##' 
-##' If there are any continuous covariates, then a single population
-##' Kaplan-Meier curve is drawn. By default, a single fitted curve is drawn
-##' with the covariates set to their mean values in the data - for categorical
-##' covariates, the means of the 0/1 indicator variables are taken.
-##' @param X Alternative way to supply covariate values, as a model matrix.
-##' See \code{\link{summary.flexsurvreg}}.  \code{newdata} is an easier way.
-##' @param type \code{"survival"} for survival, to be plotted against
-##' Kaplan-Meier estimates from \code{\link[survival]{plot.survfit}}.
-##' 
-##' \code{"cumhaz"} for cumulative hazard, plotted against transformed
-##' Kaplan-Meier estimates from \code{\link[survival]{plot.survfit}}.
-##' 
-##' \code{"hazard"} for hazard, to be plotted against smooth nonparametric
-##' estimates from \code{\link[muhaz]{muhaz}}.  The nonparametric estimates
-##' tend to be unstable, and these plots are intended just to roughly indicate
-##' the shape of the hazards through time.  The \code{min.time} and
-##' \code{max.time} options to \code{\link[muhaz]{muhaz}} may sometimes need to
-##' be passed as arguments to \code{\link{plot.flexsurvreg}} to avoid an error
-##' here.
-##' 
-##' Ignored if \code{"fn"} is specified.
-##' @param fn Custom function of the parameters to summarise against time.  The
-##' first two arguments of the function must be \code{t} representing time, and
-##' \code{start} representing left-truncation points, and any remaining
-##' arguments must be parameters of the distribution.  It should return a
-##' vector of the same length as \code{t}.
-##' @param t Vector of times to plot fitted values for, see
-##' \code{\link{summary.flexsurvreg}}.
-##' @param start Left-truncation points, see \code{\link{summary.flexsurvreg}}.
-##' @param est Plot fitted curves (\code{TRUE} or \code{FALSE}.)
-##' @param ci Plot confidence intervals for fitted curves. By default, this is
-##' \code{TRUE} if one observed/fitted curve is plotted, and \code{FALSE} if
-##' multiple curves are plotted.
-##' @param B Number of simulations controlling accuracy of confidence
-##' intervals, as used in \code{\link[=summary.flexsurvreg]{summary}}.
-##' Decrease for greater speed at the expense of accuracy, or set \code{B=0} to
-##' turn off calculation of CIs.
-##' @param cl Width of confidence intervals, by default 0.95 for 95\%
-##' intervals.
-##' @param col.obs Colour of the nonparametric curve.
-##' @param lty.obs Line type of the nonparametric curve.
-##' @param lwd.obs Line width of the nonparametric curve.
-##' @param col Colour of the fitted parametric curve(s).
-##' @param lty Line type of the fitted parametric curve(s).
-##' @param lwd Line width of the fitted parametric curve(s).
-##' @param col.ci Colour of the fitted confidence limits, defaulting to the
-##' same as for the fitted curve.
-##' @param lty.ci Line type of the fitted confidence limits.
-##' @param lwd.ci Line width of the fitted confidence limits.
-##' @param ylim y-axis limits: vector of two elements.
-##' @param add If \code{TRUE}, add lines to an existing plot, otherwise new
-##' axes are drawn.
-##' @param ... Other options to be passed to \code{\link{plot.survfit}} or
-##' \code{\link[muhaz]{muhaz}}, for example, to control the smoothness of the
-##' nonparametric hazard estimates.  The \code{min.time} and \code{max.time}
-##' options to \code{\link[muhaz]{muhaz}} may sometimes need to be changed from
-##' the defaults.
-##' @note Some standard plot arguments such as \code{"xlim","xlab"} may not
-##' work.  This function was designed as a quick check of model fit.  Users
-##' wanting publication-quality graphs are advised to set up an empty plot with
-##' the desired axes first (e.g. with \code{plot(...,type="n",...)}), then use
-##' suitable \code{\link{lines}} functions to add lines.
-##' @author C. H. Jackson \email{chris.jackson@@mrc-bsu.cam.ac.uk}
-##' @seealso \code{\link{flexsurvreg}}
-##' @keywords models hplot
-##' @export
-plot.flexsurvreg <- function(x, newdata=NULL, X=NULL, type="survival", fn=NULL, t=NULL, start=0,
-                             est=TRUE, ci=NULL, B=1000, cl=0.95,
-                             col.obs="black", lty.obs=1, lwd.obs=1,
-                             col="red",lty=1,lwd=2,
-                             col.ci=NULL,lty.ci=2,lwd.ci=1,ylim=NULL,
-                             add=FALSE,...)
-{
-    ## don't calculate or plot CIs by default if all covs are categorical -> multiple curves
-    mf <- model.frame(x)
-    Xraw <- mf[,attr(mf, "covnames.orig"), drop=FALSE]
-    if (is.null(ci))
-        ci <- ((x$ncovs == 0) || (!(sapply(Xraw,is.factor))))
-    if (!ci) B <- 0
-    summ <- summary(x, newdata=newdata, X=X, type=type, fn=fn, t=t, start=start, ci=ci, B=B, cl=cl)
-    t <- summ[[1]]$time
-    X <- if (is.null(attr(summ,"X"))) as.matrix(0, nrow=1, ncol=max(x$ncoveffs,1)) else attr(summ,"X")
-    if (is.null(col.ci)) col.ci <- col
-    if (is.null(lwd.ci)) lwd.ci <- lwd
-    dat <- x$data
-    isfac <- sapply(Xraw,is.factor)
-    if (!is.null(fn)) type <- ""
-    if (!add) {
-        mm <- as.data.frame(model.matrix(x))
-        form <- "Surv(dat$Y[,\"start\"],dat$Y[,\"stop\"],dat$Y[,\"status\"]) ~ "
-        form <- paste(form, if (x$ncovs > 0 && all(isfac)) paste("mm[,",1:x$ncoveffs,"]", collapse=" + ") else 1)
-        form <- as.formula(form)
-        ## If any continuous covariates, it is hard to define subgroups
-        ## so just plot the population survival
-        if (type=="survival") {
-            plot(survfit(form, data=mm), col=col.obs, lty=lty.obs, lwd=lwd.obs, ...)
-        }
-        else if (type=="cumhaz") {
-            plot(survfit(form, data=mm), fun="cumhaz", col=col.obs, lty=lty.obs, lwd=lwd.obs, ...)
-        }
-        else if (type=="hazard") {
-            muhaz.args <- list(...)[names(list(...)) %in% names(formals(muhaz))]
-            if (is.null(muhaz.args$min.time)) muhaz.args$min.time <- 0
-            if (is.null(muhaz.args$max.time)) muhaz.args$max.time <- with(as.data.frame(dat$Y), max(time[status==1]))
-            plot.args <- list(...)[!names(list(...)) %in% names(formals(muhaz))]
-            if (!all(dat$Y[,"start"]==0)) warning("Left-truncated data not supported by muhaz: ignoring truncation point when plotting observed hazard")
-            if (any(dat$Y[,"status"] > 1)) stop("Interval-censored data not supported by muhaz")
-            if (!all(isfac)){
-                haz <- do.call("muhaz", c(list(times=dat$Y[,"stop"], delta=dat$Y[,"status"]), muhaz.args))
-                do.call("plot", c(list(haz), list(col=col.obs, lty=lty.obs, lwd=lwd.obs), plot.args))
-            }
-            else {
-                ## plot hazard for all groups defined by unique combinations of covariates
-                group <- if(x$ncovs>0) do.call("interaction", mm) else factor(rep(0,nrow(dat$Y)))
-                Xgroup <- factor(do.call("interaction", as.data.frame(X)), levels=levels(group))
-                haz <- list()
-                for (i in 1:nrow(X)) {
-                    haz[[i]] <- do.call("muhaz", c(list(times=dat$Y[,"time"], delta=dat$Y[,"status"], subset=(group==Xgroup[i])), muhaz.args))
-                }
-                if (missing(ylim))
-                    ylim <- range(sapply(haz, function(x)range(x$haz.est)))
-                do.call("plot", c(list(haz[[1]]), list(col=col.obs, lty=lty.obs, lwd=lwd.obs, ylim=ylim), plot.args))
-                if (nrow(X)>1) {
-                    for (i in 1:nrow(X)) {
-                        lines(haz[[i]], col=col.obs, lty=lty.obs, lwd=lwd.obs)
-                    }
-                }
-            }
-        }
-    }
-    col <- rep(col, length=nrow(X)); lty=rep(lty, length=nrow(X)); lwd=rep(lwd, length=nrow(X))
-    col.ci <- rep(col.ci, length=nrow(X)); lty.ci=rep(lty.ci, length=nrow(X)); lwd.ci=rep(lwd.ci, length=nrow(X))
-    for (i in 1:nrow(X)) {
-        if (est) lines(summ[[i]]$time, summ[[i]]$est, col=col[i], lty=lty[i], lwd=lwd[i])
-        if (ci) {
-            lines(summ[[i]]$time, summ[[i]]$lcl, col=col.ci[i], lty=lty.ci[i], lwd=lwd.ci[i])
-            lines(summ[[i]]$time, summ[[i]]$ucl, col=col.ci[i], lty=lty.ci[i], lwd=lwd.ci[i])
-        }
-    }
-}
-
-
-
-##' Add fitted flexible survival curves to a plot
-##' 
-##' Add fitted survival (or hazard or cumulative hazard) curves from a
-##' \code{\link{flexsurvreg}} model fit to an existing plot.
-##' 
-##' Equivalent to \code{\link{plot.flexsurvreg}(...,add=TRUE)}.
-##' 
-##' 
-##' @param x Output from \code{\link{flexsurvreg}}, representing a fitted
-##' survival model object.
-##' @param newdata Covariate values to produce fitted curves for, as a data
-##' frame, as described in \code{\link{plot.flexsurvreg}}.
-##' @param X Covariate values to produce fitted curves for, as a matrix, as
-##' described in \code{\link{plot.flexsurvreg}}.
-##' @param type \code{"survival"} for survival, \code{"cumhaz"} for cumulative
-##' hazard, or \code{"hazard"} for hazard, as in
-##' \code{\link{plot.flexsurvreg}}.
-##' @param t Vector of times to plot fitted values for.
-##' @param est Plot fitted curves (\code{TRUE} or \code{FALSE}.)
-##' @param ci Plot confidence intervals for fitted curves.
-##' @param B Number of simulations controlling accuracy of confidence
-##' intervals, as used in \code{\link[=summary.flexsurvreg]{summary}}.
-##' @param cl Width of confidence intervals, by default 0.95 for 95\%
-##' intervals.
-##' @param col Colour of the fitted curve(s).
-##' @param lty Line type of the fitted curve(s).
-##' @param lwd Line width of the fitted curve(s).
-##' @param col.ci Colour of the confidence limits, defaulting to the same as
-##' for the fitted curve.
-##' @param lty.ci Line type of the confidence limits.
-##' @param lwd.ci Line width of the confidence limits, defaulting to the same
-##' as for the fitted curve.
-##' @param ... Other arguments to be passed to the generic \code{\link{plot}}
-##' and \code{\link{lines}} functions.
-##' @author C. H. Jackson \email{chris.jackson@@mrc-bsu.cam.ac.uk}
-##' @seealso \code{\link{flexsurvreg}}
-##' @keywords models aplot
-##' @export
-lines.flexsurvreg <- function(x, newdata=NULL, X=NULL, type="survival", t=NULL,
-                              est=TRUE, ci=NULL, B=1000, cl=0.95,
-                              col="red",lty=1,lwd=2,
-                              col.ci=NULL,lty.ci=2,lwd.ci=1, ...)
-{
-    plot.flexsurvreg(x, newdata=newdata, X=X, type=type, t=t, est=est, ci=ci, B=B, cl=cl,
-                     col=col, lty=lty, lwd=lwd, col.ci=col.ci,lty.ci=lty.ci,lwd.ci=lwd.ci, add=TRUE, ...)
-}
-
 ##' @export
 vcov.flexsurvreg <- function (object, ...)
 {
