@@ -18,6 +18,7 @@ test_that("msfit.flexsurvreg",{
 })
 
 ## With covariates
+suppressWarnings(RNGversion("3.5.0"))
 set.seed(1)
 bosms3$x <- rnorm(nrow(bosms3))
 bexp.cov <- flexsurvreg(Surv(years, status) ~ trans + x, data=bosms3, dist="exp")
@@ -37,20 +38,20 @@ test_that("Errors in msfit.flexsurvreg",{
 
 test_that("pmatrix.fs",{
     pmat <- pmatrix.fs(bexp.markov, t=c(5,10), trans=tmat)
-    expect_equal(pmat$"5"[1,2], 0.267218506920585, tol=1e-06)
+    expect_equal(pmat$"5"[1,2], 0.267218506920585, tol=1e-04)
     pmat <- pmatrix.fs(bexp.markov.cov, t=c(5,10), trans=tmat, newdata=list(x=1))
-    expect_equal(pmat$"5"[1,2], 0.259087945965485, tol=1e-06)
+    expect_equal(pmat$"5"[1,2], 0.259065437633427, tol=1e-04)
 })
 
 test_that("totlos.fs",{
     tl <- totlos.fs(bexp.markov, t=c(5), trans=tmat)
     expect_equal(as.numeric(tl), c(2.89231556324412, 0, 0, 1.06822543404334, 2.77639174263866, 0, 1.03945900271255, 2.22360825736133, 5), tol=1e-06)
     tl <- totlos.fs(bexp.markov.cov, t=c(5), trans=tmat, newdata=list(x=1))
-    expect_equal(as.numeric(tl), c(2.76046751607392, 0, 0, 1.08873482833622, 2.64545247064533, 0, 1.15079765558986, 2.35454752935467, 5))
+    expect_equal(as.numeric(tl), c(2.76115934740386, 0, 0, 1.08844199049896, 2.64568022609759, 0, 1.15039866209718, 2.35431977390241, 5))
     tl <- totlos.fs(bexp.markov, t=c(5,10), trans=tmat)
-    expect_equal(as.numeric(tl[[1]]), c(2.8923155917139, 0, 0, 1.06822541852575, 2.77639172478672,  0, 1.03945898976035, 2.22360827521328, 5))
+    expect_equal(as.numeric(tl[[1]]), c(2.89231557124359, 0, 0, 1.06822540869797, 2.77639177178247, 0, 1.03945902005845, 2.22360822821753, 5))
     tl <- totlos.fs(bexp.markov.cov, t=c(5,10), trans=tmat, newdata=list(x=1))
-    expect_equal(as.numeric(tl[[1]]),c(2.76046751607392, 0, 0, 1.08873482833622, 2.64545247064533, 0, 1.15079765558986, 2.35454752935467, 5))
+    expect_equal(as.numeric(tl[[1]]),c(2.76115934740386, 0, 0, 1.08844199049896, 2.64568022609759, 0, 1.15039866209718, 2.35431977390241, 5))
     attr(tl, "P")
 })
 
@@ -93,6 +94,14 @@ test_that("list format in output functions", {
     pmatrix.fs(bweim.list, t=5, trans=tmat)
     pmatrix.fs(bweim.list, t=c(5,10), trans=tmat)
 
+   
+    pmat1 <- pmatrix.fs(bweic.list, t=c(5,10), trans=tmat, newdata=list(x=-1))
+    pmat3 <- pmatrix.fs(bweic.list, t=c(5,10), trans=tmat, newdata=list(x=c(-1,-1,-1)))
+    expect_equal(pmat1[[1]], pmat3[[1]])
+
+    expect_error(pmatrix.fs(bweic.list, t=c(5,10), trans=tmat, newdata=list(x=c(-1,-1))),
+                 "must either have one row, or one row for each of the 3 allowed transitions")
+
     pmatrix.fs(bln.markov, t=5, trans=tmat)
 
     totlos.fs(bweim.list, t=5, trans=tmat)
